/********************************************************************
 * Copyright  2004 VHA. All rights reserved
 ********************************************************************/

package gov.va.med.fw.hl7.segment;

// Java classes
import java.util.List;

// Framework classes
import gov.va.med.fw.hl7.InvalidSegmentException;
import gov.va.med.fw.hl7.Segment;
import gov.va.med.fw.hl7.constants.SegmentConstants;

/**
 * A QRF segment containing information about batch trailer
 * 
 * @author Alex Yoon
 * @version 1.0
 */

public class QRF extends Segment {

    /**
	 * An instance of serialVersionUID
	 */
	private static final long serialVersionUID = -6376833977294690188L;

	/**
     * Constructs an empty segment. This constructor is ususally usd by derived
     * segment classes to populate its own segment data.
     */
    public QRF() {
        super();
        setName(SegmentConstants.QRF);
    }

    /**
     * Constructs a segment of elements from a raw data string. This constructor
     * is usually used to construct an inbound message from a raw message's
     * segment data.
     * 
     * @param data
     *            A raw data string
     * @throws InvalidSegmentException
     *             Thrown if failed to construct a segment due to missing
     *             parameters or invalid segment data to parse.
     */
    public QRF(String data) throws InvalidSegmentException {
        super(data);
    }

    /**
     * Constructs a segment of elements from a raw data string, an array of
     * valid encoding character strings, and a element delimiter string. This
     * constructor is usually used to construct an inbound message from a raw
     * message's segment data, a specific encoding characters, and a field
     * delimiter.
     * 
     * @param encodingChars
     *            An array of encoding characters
     * @param elementDelimiter
     *            A field delimiter
     * @param segmentData
     *            A raw data string
     * @throws InvalidSegmentException
     *             Thrown if failed to construct a segment due to missing
     *             parameters or invalid segment data to parse.
     */
    public QRF(String[] encodingChars, String elementDelimiter,
            String segmentData) throws InvalidSegmentException {
        super(encodingChars, elementDelimiter, segmentData);
    }

    /**
     * Constructs a segment from a segment name and a list of elements. This
     * constructor is usually used to construct an outbound message from a list
     * of specific segment elements, and a segment name.
     * 
     * @param name
     *            A segment name
     * @param elements
     *            A list of segment elements
     * @throws InvalidSegmentException
     *             Thrown if failed to construct a segment due to missing
     *             parameters or invalid segment data to parse.
     */
    public QRF(String name, List elements) throws InvalidSegmentException {
        super(name, elements);
    }

    /**
     * Constructs a segment from an array of encoding characters, a field
     * delimiter, a segment name, and a list of field data. This constructor is
     * usually used to construct an outbound message from a list of specific
     * segment elements, element delimiter, and encoding characters.
     * 
     * @param encodingChars
     *            An array of encoding characters
     * @param elementDelimiter
     *            A field delimiter
     * @param segmentName
     *            A segment name
     * @param segmentElements
     *            A list of field data
     * @throws InvalidSegmentException
     *             Thrown if failed to construct a segment due to missing
     *             parameters or invalid segment data to parse.
     */
    public QRF(String[] encodingChars, String elementDelimiter,
            String segmentName, List segmentElements)
            throws InvalidSegmentException {
        super(encodingChars, elementDelimiter, segmentName, segmentElements);
    }

    /**
     * 
     * "IVM" (IVM Center)
     * 
     * @param input
     *            A value of WhereSubjectFilter
     */
    public void setWhereSubjectFilter(String input) {
        setElement(input, 0);
    }

    /**
     * 
     * "IVM" (IVM Center)
     * 
     * @return String A value of WhereSubjectFilter
     */
    public String getWhereSubjectFilter() {
        return getElement(0);
    }

    /**
     * 
     * NOT USED (VISTA Messaging HL7)
     * 
     * @param input
     *            A value of WhenDataStartDate
     */
    public void setWhenDataStartDate(String input) {
        setElement(input, 1);
    }

    /**
     * 
     * NOT USED (VISTA Messaging HL7)
     * 
     * @return String A value of WhenDataStartDate
     */
    public String getWhenDataStartDate() {
        return getElement(1);
    }

    /**
     * 
     * NOT USED (VISTA Messaging HL7)
     * 
     * @param input
     *            A value of WhenDataEndDate
     */
    public void setWhenDataEndDate(String input) {
        setElement(input, 2);
    }

    /**
     * 
     * NOT USED (VISTA Messaging HL7)
     * 
     * @return String A value of WhenDataEndDate
     */
    public String getWhenDataEndDate() {
        return getElement(2);
    }

    /**
     * 
     * File #2 Field .03 (DATE OF BIRTH) (VISTA Messaging HL7)
     * 
     * @param input
     *            A value of WhatUserQualifier
     */
    public void setWhatUserQualifier(String input) {
        setElement(input, 3);
    }

    /**
     * 
     * File #2 Field .03 (DATE OF BIRTH) (VISTA Messaging HL7)
     * 
     * @return String A value of WhatUserQualifier
     */
    public String getWhatUserQualifier() {
        return getElement(3);
    }

    /**
     * 
     * File #2 Field .02 (SEX) (VISTA Messaging HL7)
     * 
     * @param input
     *            A value of OtherQuerySubjectFilter
     */
    public void setOtherQuerySubjectFilter(String input) {
        setElement(input, 4);
    }

    /**
     * 
     * File #2 Field .02 (SEX) (VISTA Messaging HL7)
     * 
     * @return String A value of OtherQuerySubjectFilter
     */
    public String getOtherQuerySubjectFilter() {
        return getElement(4);
    }
}