package gov.va.med.esr.voa.validator;

import gov.va.med.esr.common.model.lookup.Country;
import gov.va.med.esr.common.model.lookup.EmploymentStatus;
import gov.va.med.esr.common.model.lookup.MaritalStatus;
import gov.va.med.esr.common.model.lookup.VAFacility;
import gov.va.med.esr.common.model.lookup.VAFacilityType;
import gov.va.med.esr.voa.common.ErrorCode;
import gov.va.med.esr.voa.util.ServiceUtils;
import gov.va.med.esr.voa.webservice.AddressCollection;
import gov.va.med.esr.voa.webservice.AddressInfo;
import gov.va.med.esr.voa.webservice.ApplicationCollection;
import gov.va.med.esr.voa.webservice.ApplicationInfo;
import gov.va.med.esr.voa.webservice.ContactInfo;
import gov.va.med.esr.voa.webservice.DemographicInfo;
import gov.va.med.esr.voa.webservice.EeSummary;
import gov.va.med.esr.voa.webservice.EmploymentInfo;
import gov.va.med.esr.voa.webservice.EnrollmentDeterminationInfo;
import gov.va.med.esr.voa.webservice.FinancialStatementInfo;
import gov.va.med.esr.voa.webservice.FinancialsInfo;
import gov.va.med.esr.voa.webservice.Form;
import gov.va.med.esr.voa.webservice.FormIdentifier;
import gov.va.med.esr.voa.webservice.InsuranceCollection;
import gov.va.med.esr.voa.webservice.InsuranceInfo;
import gov.va.med.esr.voa.webservice.MilitaryServiceEpisodeCollection;
import gov.va.med.esr.voa.webservice.MilitaryServiceEpisodeInfo;
import gov.va.med.esr.voa.webservice.MilitaryServiceInfo;
import gov.va.med.esr.voa.webservice.MilitaryServiceSiteRecordCollection;
import gov.va.med.esr.voa.webservice.MilitaryServiceSiteRecordInfo;
import gov.va.med.esr.voa.webservice.PersonInfo;
import gov.va.med.esr.voa.webservice.SpouseFinancialsCollection;
import gov.va.med.esr.voa.webservice.SpouseFinancialsInfo;
import gov.va.med.esr.voa.webservice.SpouseInfo;

import java.util.Iterator;
import java.util.List;

import javax.xml.datatype.XMLGregorianCalendar;

import org.springframework.validation.Errors;
import org.springframework.validation.ValidationUtils;

/**
 * Form validation
 */
public class FormValidator extends AbstractValidator {

	// 1010EZ form type
	private static final String TYPE_1010EZ = "100";

	// CCR 13857 - 21526EZ form type
	private static final String TYPE_21526EZ = "105";

	// personInformation validator
	private static final PersonInfoValidator personInfoValidator = new PersonInfoValidator();

	// demographicInformation validator
	private static final DemographicInfoValidator demographicInfoValidator = new DemographicInfoValidator();

	// enrollment determination information validator
	private static final EnrollmentDeterminationInfoValidator enrollmentDeterminationInfoValidator = new EnrollmentDeterminationInfoValidator();


	@SuppressWarnings("rawtypes")
	@Override
	public boolean supports(Class clazz) {
		return Form.class.equals(clazz);
	}

	@Override
	public void validate(Object target, Errors errors) {
		 if (logger.isDebugEnabled()){
			 logger.debug("validate form.");
		 }

		if (target != null) {
			Form form = (Form) target;
			EeSummary eeSummary = form.getSummary();
			if (eeSummary != null) {
				PersonInfo personInfo = eeSummary.getPersonInfo();
				if (personInfo != null) {
					ValidationUtils.invokeValidator(personInfoValidator,
							personInfo, errors);
				}

				DemographicInfo demographicInfo = eeSummary.getDemographics();

				//CCR 13857 - auto-enrollment from VBA, auto-set PF to nearest in form data
				if (form.getFormIdentifier().getType().equalsIgnoreCase(TYPE_21526EZ)) {

					AddressInfo addressInfo = null;
					String zipCode = null;
					ContactInfo contactInfo = demographicInfo.getContactInfo();

					if (contactInfo != null) {
						AddressCollection addressCollection = contactInfo
								.getAddresses();
						if (addressCollection != null) {
							List<AddressInfo> addressInfoList = addressCollection.getAddress();
							if (!addressInfoList.isEmpty())
								addressInfo  = addressInfoList.get(0);
							}
					}

					if (addressInfo != null) {
						 zipCode = addressInfo.getZipCode();
						 try {
							 VAFacility nearest = findNearestFacilityByZip(zipCode);
							 if (nearest != null) {
								 demographicInfo.setPreferredFacility(nearest.getStationNumber());
							 } else {
								 logger.info("VOA 526 PF nearest facility is Null for zip code"); // + zipCode);
								 errors.reject(ErrorCode.VOA_0021, "Nearest VA Facility cannot be determined from zip code.");
							 }
						 } catch (Exception e) {
							 logger.info("VOA 526 PF failed for Exception: " + e.getMessage());
							 errors.reject(ErrorCode.VOA_0021, "Nearest VA Facility cannot be determined from zip code.");
						 }
					} else {
						logger.info("VOA 526 PF failed for Null AddressInfo");
						errors.reject(ErrorCode.VOA_0021, "Nearest VA Facility cannot be determined from zip code.");
					}
				}


				if (demographicInfo != null) {
					ValidationUtils.invokeValidator(demographicInfoValidator,
							demographicInfo, errors);
				}

				EnrollmentDeterminationInfo enrollmentDeterminationInfo = eeSummary
						.getEnrollmentDeterminationInfo();
				if (enrollmentDeterminationInfo != null) {
					ValidationUtils.invokeValidator(
							enrollmentDeterminationInfoValidator,
							enrollmentDeterminationInfo, errors);
				}

				//CCR 13311: remove validation for NOK & EC
/*				AssociationCollection associationCollection = eeSummary
						.getAssociations();
				if (associationCollection != null) {
					List<AssociationInfo> associationInfoList = associationCollection
							.getAssociation();
					for (AssociationInfo associationInfo : associationInfoList) {
						//CCR 12852 - Even though NOK & EC are not supposed to be there,
						//but VOA WS should not cause the error messages when they are present for anonymous and level 1
						AssociationInfoValidator associationValidatior = new AssociationInfoValidator();
						associationValidatior.setNewOMBApprovedForm(this.isNewOMBApprovedForm);
						//ValidationUtils.invokeValidator(
								//new AssociationInfoValidator(),
								//associationInfo, errors);
						ValidationUtils.invokeValidator(
								associationValidatior,
								associationInfo, errors);
					}
				}*/

				InsuranceCollection insuranceCollection = eeSummary
						.getInsuranceList();
				if (insuranceCollection != null) {
					List<InsuranceInfo> insuranceInfoList = insuranceCollection
							.getInsurance();
					for (InsuranceInfo insuranceInfo : insuranceInfoList) {
						ValidationUtils.invokeValidator(
								new InsuranceInfoValidator(), insuranceInfo,
								errors);
					}
				}

				//CCR 13311 remove veteran employment
/*				EmploymentInfo employmentInfo = eeSummary.getEmploymentInfo();
				if (employmentInfo != null) {
					ValidationUtils.invokeValidator(
							new EmploymentInfoValidator(), employmentInfo,
							errors);
				}*/

				MilitaryServiceInfo militaryServiceInfo = eeSummary
						.getMilitaryServiceInfo();
				if (militaryServiceInfo != null) {
					MilitaryServiceSiteRecordCollection militaryServiceSiteRecordCollection = militaryServiceInfo
							.getMilitaryServiceSiteRecords();
					if (militaryServiceSiteRecordCollection != null) {
						List<MilitaryServiceSiteRecordInfo> militaryServiceSiteRecordInfoList = militaryServiceSiteRecordCollection
								.getMilitaryServiceSiteRecord();
						for (MilitaryServiceSiteRecordInfo militaryServiceSiteRecordInfo : militaryServiceSiteRecordInfoList) {
							MilitaryServiceEpisodeCollection militaryServiceEpisodeCollection = militaryServiceSiteRecordInfo
									.getMilitaryServiceEpisodes();
							if (militaryServiceEpisodeCollection != null) {
								List<MilitaryServiceEpisodeInfo> militaryServiceEpisodeInfoList = militaryServiceEpisodeCollection
										.getMilitaryServiceEpisode();
								for (MilitaryServiceEpisodeInfo militaryServiceEpisodeInfo : militaryServiceEpisodeInfoList) {
									ValidationUtils
											.invokeValidator(
													new MilitaryServiceEpisodeInfoValidator(),
													militaryServiceEpisodeInfo,
													errors);
								}
							}
						}
					}
				}

				FinancialsInfo financialsInfo = eeSummary.getFinancialsInfo();
				if (financialsInfo != null) {
					//CCR 13311 remove spouse employment
/*					// validate spouse employment
					validateSpouseEmploymentInfo(demographicInfo,
							financialsInfo, errors);*/

					// validate veteran financial info
					ValidationUtils.invokeValidator(
							new FinancialInfoValidator(), financialsInfo,
							errors);
				}
			}

			ApplicationCollection applicationCollection = form.getApplications();
			validateApplicationInfo(applicationCollection, errors);

			validateAppDate(form, errors);

			 if (logger.isDebugEnabled()){
				 logger.debug("validate form - Done:" +  form);
			 }
		}
	}

	//CCR13857 - auto-enrollment from VBA, find closest logical zip code
	private VAFacility findNearestFacilityByZip(String zip) throws Exception {

		if (zip == null) return null;

		List facilities = getLookupService().getAllVaFacilitys();

		boolean match = false;
		int level = 5;
		VAFacility nearest = null;

		//walk back to single digit if needed
		for (int i = 0; i < 5; i++) {
			if (match) break;

			zip = zip.substring(0, level);

			Iterator itr = facilities.iterator();

			//look for zip match on current level for any OPC, VAMC or CBOC that is active
			while (itr != null && itr.hasNext()) {
				VAFacility vamc = (VAFacility)itr.next();

				if (vamc != null &&  vamc.getEndDate() == null && vamc.getType().getIsMedicalTreating()) {
					if (vamc.getType().getCode().equals(VAFacilityType.CODE_OPC.getName()) || vamc.getType().getCode().equals(VAFacilityType.CODE_CBOC.getName())
							|| vamc.getType().getCode().equals(VAFacilityType.CODE_VAMC.getName())) {

						if (vamc.getStreetAddress() != null && vamc.getStreetAddress().getZipCode() != null) {
							String vaZip = vamc.getStreetAddress().getZipCode().substring(0, level);
							if (zip.equals(vaZip)) {
								match = true;
								nearest = vamc;
								logger.info("setting VOA form 526 PF for zip code level match:" + vaZip + " to site:" + vamc.getStationNumber());
								break;
							}
						}
					}
				}
			}
			level--;
		}

		return nearest;
	}

	/**
	 * Validate spouse employment information. Because it depends on veteran's
	 * maritial status, it is handled differently.
	 *
	 * @param demographicInfo
	 * @param financialsInfo
	 * @param errors
	 */
	private void validateSpouseEmploymentInfo(DemographicInfo demographicInfo,
			FinancialsInfo financialsInfo, Errors errors) {
		if (demographicInfo != null) {
			String maritialStatus = demographicInfo.getMaritalStatus();
			if (MaritalStatus.CODE_MARRIED.getCode().equalsIgnoreCase(
					maritialStatus)
					|| MaritalStatus.CODE_SEPARATED.getCode().equalsIgnoreCase(
							maritialStatus)) {
				if (financialsInfo != null) {
					FinancialStatementInfo financialStatementInfo = financialsInfo
							.getFinancialStatement();
					if (financialStatementInfo != null) {
						SpouseFinancialsCollection spouseFinancialsCollection = financialStatementInfo
								.getSpouseFinancialsList();
						if (spouseFinancialsCollection != null) {
							List<SpouseFinancialsInfo> SpouseFinancialsInfoList = spouseFinancialsCollection
									.getSpouseFinancials();
							// Even it is a collection, for 1010EZ, it has only
							// one spouse.
							for (SpouseFinancialsInfo spouseFinancialsInfo : SpouseFinancialsInfoList) {
								SpouseInfo spouseInfo = spouseFinancialsInfo
										.getSpouse();
								validateSpouseEmployment(spouseInfo, errors);
							}
						}
					}
				}
			}
		}
	}

	/**
	 * Validate spouse employment information
	 *
	 * @param spouseInfo
	 * @param errors
	 */
	private void validateSpouseEmployment(SpouseInfo spouseInfo, Errors errors) {
		if (spouseInfo != null) {
			EmploymentInfo employmentInfo = spouseInfo.getEmployment();
			if (employmentInfo != null) {

				String employmentStatus = employmentInfo.getEmploymentStatus();
				if (org.apache.commons.lang.StringUtils.isBlank(employmentStatus)) {
					//CCR12789
					if(!this.isNewOMBApprovedForm){
						errors.reject(ErrorCode.VOA_0089,
							"Missing required field: Spouse's employment status.");
					}
				}

				if (EmploymentStatus.CODE_RETIRED.getCode().equalsIgnoreCase(
						employmentStatus)) {
					String retirementDate = employmentInfo.getRetirementDate();
					if (org.apache.commons.lang.StringUtils.isBlank(retirementDate)) {
						errors.reject(ErrorCode.VOA_0090,
								"Missing required field: Spouse's date of retirement.");
					} else {
						boolean isValidDate = ServiceUtils
								.validateDate(retirementDate);
						boolean isValidYear = ServiceUtils
								.isValidYear(retirementDate);
						if (isValidDate || isValidYear) {
							boolean isFutureDate = false;
							if (isValidDate) {
								isFutureDate = ServiceUtils
										.isFutureDate(retirementDate);
							} else if (isValidYear) {
								isFutureDate = ServiceUtils
										.isFutureYear(retirementDate);
							}
							if (isFutureDate) {
								errors.reject(ErrorCode.VOA_0091,
										"Invalid date of retirement for spouse.");
								 if (logger.isDebugEnabled()){
									 logger.debug("Invalid date of retirement for spouse.");
								 }
							}
						} else {
							errors.reject(ErrorCode.VOA_0091,
									"Invalid date of retirement for spouse.");
							 if (logger.isDebugEnabled()){
								 logger.debug("Invalid date of retirement for spouse.");
							 }
						}
					}
				}

				// only when spouse is employed
				if (EmploymentStatus.CODE_FULL_TIME.getCode().equalsIgnoreCase(
						employmentStatus)
						|| EmploymentStatus.CODE_PART_TIME.getCode()
								.equalsIgnoreCase(employmentStatus)) {
					String employmentName = employmentInfo.getEmployerName();
					if (org.apache.commons.lang.StringUtils.isBlank(employmentName)) {
						errors.reject(ErrorCode.VOA_0092,
								"Missing required field: Name of Spouse's employer.");
					} else {
						boolean isValid = ServiceUtils
								.hasLessThan30Chars(employmentName);
						if (!isValid) {
							errors.reject(ErrorCode.VOA_0093,
									"Invalid value for the Name of Spouse's employer.");
						}
					}

					AddressInfo addressInfo = employmentInfo
							.getEmployerAddress();

					if (addressInfo == null) {
						errors.reject(ErrorCode.VOA_0094,
								"Missing required field: Street address of Spouse's employer.");
					} else {
						validateSpouseEmploymentAddress(addressInfo, errors);
					}

					String phoneNumber = employmentInfo.getEmployerPhone();
					if (phoneNumber != null) {
						boolean isValid = ServiceUtils
								.validatePhoneWithExtension(phoneNumber);
						if (!isValid) {
							errors.reject(ErrorCode.VOA_0102,
									"Invalid phone number for Spouse's employer.");
						}
					}
				}
			}
		}
	}

	private void validateSpouseEmploymentAddress(AddressInfo addressInfo,
			Errors errors) {
		 if (logger.isDebugEnabled()){
			 logger.debug("AddressInfo validation...");
		 }

		String line1 = addressInfo.getLine1();
		if (org.apache.commons.lang.StringUtils.isBlank(line1)) {
			errors.reject(ErrorCode.VOA_0094,
					"Missing required field: Street address of Spouse's employer.");
		} else if (!ServiceUtils.hasLessThan30Chars(line1)) {
			errors.reject(ErrorCode.VOA_0095,
					"Invalid value for Street address of Spouse's employer.");
		}

		String city = addressInfo.getCity();
		if (org.apache.commons.lang.StringUtils.isBlank(city)) {
			errors.reject(ErrorCode.VOA_0096,
					"Missing required field: City of Spouse's employer.");
		} else {
			if (!ServiceUtils.hasLessThan30Chars(city)) {
				errors.reject(ErrorCode.VOA_0097,
						"Invalid value for City of Spouse's employer.");
			}
		}

		String country = addressInfo.getCountry();
		if (org.apache.commons.lang.StringUtils.isBlank(country)) {
			errors.reject(ErrorCode.VOA_0237,
					"Missing required field: Country of Spouse's employer.");
			 if (logger.isDebugEnabled()){
				 logger.debug("Missing required field: Country of Spouse's employer. ");
			 }
		} else {
			try {
				getLookupService().getCountryByCode(country);
			} catch (Exception e) {
				errors.reject(ErrorCode.VOA_0245,
						"Invalid country for Spouse's employer.");
				 if (logger.isDebugEnabled()){
					logger.debug("Invalid country for Spouse's employer. "
							+ e.getMessage());
				 }
			}

			String stateProvince = null;
			if (Country.CODE_USA.getCode().equalsIgnoreCase(country)) {
				stateProvince = addressInfo.getState();

				String zip = addressInfo.getZipCode();
				if (org.apache.commons.lang.StringUtils.isBlank(zip)) {
					errors.reject(ErrorCode.VOA_0101,
							"Missing required field: Zip code for Spouse's employer.");
				} else {
					boolean isValid = ServiceUtils.validateZip(zip);
					if (!isValid) {
						errors.reject(ErrorCode.VOA_0100,
								"Invalid zip code for Spouse's employer.");
					}
				}

				String zipPlus = addressInfo.getZipPlus4();
				if (org.apache.commons.lang.StringUtils.isNotBlank(zipPlus)) {
					boolean isValid = ServiceUtils.validateZipPlus(zipPlus);
					if (!isValid) {
						errors.reject(ErrorCode.VOA_0235,
								"Invalid zip code plus 4 for Spouse's employer.");
					}
				}
			} else if (Country.CODE_CAN.getCode().equalsIgnoreCase(country)
					|| Country.CODE_MEX.getCode().equalsIgnoreCase(country)) {
				stateProvince = addressInfo.getProvinceCode();
			}

			// only check state of USA or province of CAN & MEX
			if (Country.CODE_USA.getCode().equalsIgnoreCase(country)
					|| Country.CODE_CAN.getCode().equalsIgnoreCase(country)
					|| Country.CODE_MEX.getCode().equalsIgnoreCase(country)) {
				if (org.apache.commons.lang.StringUtils.isBlank(stateProvince)) {
					errors.reject(ErrorCode.VOA_0099,
							"Missing required field: State or Pronince for Spouse's employer.");
					 if (logger.isDebugEnabled()){
						 logger.debug("Missing required field: State or Pronince for Spouse's employer.");
					 }
				} else {
					try {
						getLookupService().getStateByCode(stateProvince);
					} catch (Exception e) {
						errors.reject(ErrorCode.VOA_0098,
								"Invalid value for State or Province for Spouse's employer.");
						 if (logger.isDebugEnabled()){
							 logger.debug("Invalid value for State or Province for Spouse's employer. "
								+ e.getMessage());
						 }
					}
				}
			}
		}
		 if (logger.isDebugEnabled()){
			 logger.debug("AddressInfo validation - Done.");
		 }
	}

	/**
	 * VOA_0018: Neither Health Benefits or Dental is indicated, at least one is
	 * required; both may be indicated.
	 *
	 * TODO: To support both, it is under discussion. The schema need to be
	 * modified and applicationInfo should have 2 benefitTypes.
	 *
	 * @param applicationCollection
	 * @param errors
	 */
	private void validateApplicationInfo(
			ApplicationCollection applicationCollection, Errors errors) {
		if (applicationCollection != null) {
			List<ApplicationInfo> applicationInfoList = applicationCollection
					.getApplicationInfo();
			for (ApplicationInfo applicationInfo : applicationInfoList) {
				String benefitType = applicationInfo.getBenefitType();
				//CCR12789
				//CCR 13311 benefitType removed from 1010EZR
/*				if(!this.isNewOMBApprovedForm)
				{
					if (StringUtils.isBlank(benefitType)) {
						errors.reject(ErrorCode.VOA_0018,
								"Missing Required Field: Type of benefit applied for.");
						logger.debug("Missing Required Field: Type of benefit applied for.");
					}
				}	*/
			}
		}
	}

	/**
	 * application Date is a required field for 1010EZ form.
	 * also in 21526EZ form per CCR13857
	 * @param form
	 */
	private void validateAppDate(Form form, Errors errors) {
		FormIdentifier formIdentifier = form.getFormIdentifier();
		if (formIdentifier != null) {
			String formType = formIdentifier.getType();
			//validate 1010EZ and
			//also newly created type 21526EZ per CCR13857
			if (TYPE_1010EZ.equalsIgnoreCase(formType) || TYPE_21526EZ.equalsIgnoreCase(formType) ) {
				XMLGregorianCalendar appDate = null;
				ApplicationCollection applicationCollection = form.getApplications();
				if (applicationCollection != null) {
					List<ApplicationInfo> applicationInfoList = applicationCollection.getApplicationInfo();
					for (ApplicationInfo applicationInfo : applicationInfoList) {
						XMLGregorianCalendar temp = applicationInfo.getAppDate();
						if (temp != null) {
							appDate = temp;
							break;
						}
					}
				}

				if (appDate == null) {
				   if (TYPE_1010EZ.equalsIgnoreCase(formType)) {
					errors.reject(ErrorCode.VOA_0248,
							"Missing required field for 10-10EZ Application:  Application Date.");
					 if (logger.isDebugEnabled()){
				   		 logger.debug("Missing required field for 10-10EZ Application:  Application Date.");
				   }
				  }
				  else if (TYPE_21526EZ.equalsIgnoreCase(formType)) {  //added for newly created type 21526EZ per CCR13857
							errors.reject(ErrorCode.VOA_0253,
									"Missing required field for 21526EZ Application:  Application Date.");
					 if (logger.isDebugEnabled()){
						 logger.debug("Missing required field for 21526EZ Application:  Application Date.");
					 }
				 }
				}
			}
		}
	}

}
